unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, K8074, CPort, ExtCtrls, jpeg, IniFiles;

type
  TMainForm = class(TForm)
    lblPort: TLabel;
    cbPort: TComboBox;
    btnConnect: TButton;
    btnDisconnect: TButton;
    grpAddress: TGroupBox;
    cbAddress: TComboBox;
    btnShowAddr: TButton;
    cbNewAddress: TComboBox;
    btnSetAddress: TButton;
    grpOnOff: TGroupBox;
    btnSetAll: TButton;
    btnClearAll: TButton;
    grpToggle: TGroupBox;
    btnEmergencyStop: TButton;
    grpMomentary: TGroupBox;
    MomTimer: TTimer;
    Image1: TImage;
    btnReset: TButton;
    procedure btnEmergencyStopClick(Sender: TObject);
    procedure btnSetAllClick(Sender: TObject);
    procedure btnClearAllClick(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnDisconnectClick(Sender: TObject);
    procedure btnSetAddressClick(Sender: TObject);
    procedure btnSetClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnToggleClick(Sender: TObject);
    procedure btnShowAddrClick(Sender: TObject);
    procedure btnMomentaryMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnMomentaryMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MomTimerTimer(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
  public
    hDevice: THandle;
    btnClear:     array[0..7] of TButton;
    btnSet:       array[0..7] of TButton;
    btnToggle:    array[0..7] of TButton;
    btnMomentary: array[0..7] of TButton;
    fMomIdx: Integer;
    procedure UpdateConnectionState(Connected: Boolean);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}


procedure TMainForm.btnEmergencyStopClick(Sender: TObject);
begin
  EmergencyStop(hDevice);
end;

procedure TMainForm.btnSetAllClick(Sender: TObject);
begin
  SendByte(hDevice, (cbAddress.ItemIndex+1), $FF);
end;

procedure TMainForm.btnClearAllClick(Sender: TObject);
begin
  SendByte(hDevice, (cbAddress.ItemIndex+1), $00);
end;

procedure TMainForm.UpdateConnectionState(Connected: Boolean);
begin
  grpToggle.Enabled   := Connected;
  grpAddress.Enabled    := Connected;
  grpMomentary.Enabled := Connected;
  grpOnOff.Enabled  := Connected;
  cbPort.Enabled        := not Connected;
  btnConnect.Visible    := not Connected;
  btnDisconnect.Visible := Connected;
  btnEmergencyStop.Enabled := Connected;
end;

procedure TMainForm.btnConnectClick(Sender: TObject);
begin
  hDevice := OpenDevice(PChar(cbPort.Text));
  if (hDevice <> 0) then
    UpdateConnectionState(True);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  UpdateConnectionState(False);

  for I:=1 to 255 do
    cbAddress.Items.Add(IntToStr(I));
  cbNewAddress.Items.AddStrings(cbAddress.Items);

  // default address
  cbAddress.ItemIndex := 0;
  cbNewAddress.ItemIndex := cbAddress.ItemIndex;

  for I:=0 to 7 do
  begin
    btnSet[I] := TButton.Create(Self);
    btnSet[I].Parent := grpOnOff;
    btnSet[I].Top := 16;
    btnSet[I].Left := 8 + i * 32;
    btnSet[I].Caption := IntToStr(I+1);
    btnSet[I].Tag := I + 1;
    btnSet[I].Width := 25;
    btnSet[I].Height := 25;
    btnSet[I].OnClick := btnSetClick;

    btnClear[I] := TButton.Create(Self);
    btnClear[I].Parent := grpOnOff;
    btnClear[I].Top := 40;
    btnClear[I].Left := 8 + i * 32;
    btnClear[I].Caption := '0';
    btnClear[I].Tag := I + 1;
    btnClear[I].Width := 25;
    btnClear[I].Height := 25;
    btnClear[I].OnClick := btnClearClick;

    btnToggle[I] := TButton.Create(Self);
    btnToggle[I].Parent := grpToggle;
    btnToggle[I].Top := 16;
    btnToggle[I].Left := 8 + i * 32;
    btnToggle[I].Caption := IntToStr(I+1);
    btnToggle[I].Tag := I + 1;
    btnToggle[I].Width := 25;
    btnToggle[I].Height := 25;
    btnToggle[I].OnClick := btnToggleClick;

    btnMomentary[I] := TButton.Create(Self);
    btnMomentary[I].Parent := grpMomentary;
    btnMomentary[I].Top := 16;
    btnMomentary[I].Left := 8 + i * 32;
    btnMomentary[I].Caption := IntToStr(I+1);
    btnMomentary[I].Tag := I + 1;
    btnMomentary[I].Width := 25;
    btnMomentary[I].Height := 25;
    btnMomentary[I].OnMouseDown := btnMomentaryMouseDown;
    btnMomentary[I].OnMouseUp := btnMomentaryMouseUp;    
  end;
end;

procedure TMainForm.btnDisconnectClick(Sender: TObject);
begin
  CloseDevice(hDevice);
  UpdateConnectionState(False);
end;

procedure TMainForm.btnSetAddressClick(Sender: TObject);
begin
  SetAddress(hDevice, (cbAddress.ItemIndex+1), (cbNewAddress.ItemIndex+1));
  cbAddress.ItemIndex := cbNewAddress.ItemIndex;
end;

procedure TMainForm.btnSetClick(Sender: TObject);
begin
  SetRelay(hDevice, (cbAddress.ItemIndex+1), TButton(Sender).Tag);
end;

procedure TMainForm.btnClearClick(Sender: TObject);
begin
  ClearRelay(hDevice, (cbAddress.ItemIndex+1), TButton(Sender).Tag);
end;

procedure TMainForm.btnToggleClick(Sender: TObject);
begin
  ToggleRelay(hDevice, (cbAddress.ItemIndex+1), TButton(Sender).Tag);
end;

procedure TMainForm.btnShowAddrClick(Sender: TObject);
begin
  DisplayAddress(hDevice);
end;

procedure TMainForm.btnMomentaryMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  MomTimer.Interval := IniFile.ReadInteger('Settings', 'MomentaryInterval', MomTimer.Interval);
  FreeAndNil(IniFile);

  fMomIdx := TButton(Sender).Tag;
  MomTimer.Enabled := True;
end;

procedure TMainForm.btnMomentaryMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MomTimer.Enabled := False;
end;

procedure TMainForm.MomTimerTimer(Sender: TObject);
begin
  SendCommand(hDevice, (cbAddress.ItemIndex+1), $50, 48+fMomIdx); // 50h = 80d = 'P'
end;

procedure TMainForm.btnResetClick(Sender: TObject);
begin
  if mrYes = MessageDlg('Are you sure? Note that all 8 channel' +
    ' relay cards under power and within range will be reset to address 1', mtWarning, [mbYes, mbNo], 0) then
  begin
    ForceAddressReset(hDevice);
    cbAddress.ItemIndex := 0;
  end;
end;

end.
