using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Velleman.Kits;

namespace K8074.NET
{
    public partial class MainForm : Form
    {
        private int m_device;
        private int m_MomIdx;

        private Button[] btnToggle = new Button[8];
        private Button[] btnSwitchOn = new Button[8];
        private Button[] btnSwitchOff = new Button[8];
        private Button[] btnMomentary = new Button[8];

        public MainForm()
        {
            InitializeComponent();
        }

        private void updateConnectionState(bool connected)
        {
            btnConnect.Visible = !connected;
            btnDisconnect.Visible = connected;
            cbPort.Enabled = !connected;
            grpMomentary.Enabled = connected;
            grpToggle.Enabled = connected;
            grpOnOff.Enabled = connected;
            grpAddress.Enabled = connected;
            btnEmergencyStop.Enabled = connected;
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            m_device = K8074Native.OpenDevice(cbPort.Text);
            if (m_device == 0)
                MessageBox.Show("Unable to connect", "Connect", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                this.updateConnectionState(true);
            }
        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            K8074Native.CloseDevice(m_device);
            this.updateConnectionState(false);
        }

        private void btnEmergencyStop_Click(object sender, EventArgs e)
        {
            K8074Native.EmergencyStop(m_device);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            for (int i = 1; i <= 255; i++)
            {
                cbAddress.Items.Add(i.ToString());
                cbNewAddress.Items.Add(i.ToString());
            }
            cbAddress.SelectedIndex = 0;
            cbNewAddress.SelectedIndex = cbAddress.SelectedIndex;

            for (int i = 0; i < 8; i++)
            {
                btnSwitchOn[i] = new Button();
                btnSwitchOn[i].Parent = grpOnOff;
                btnSwitchOn[i].Width = 25;
                btnSwitchOn[i].Height = 25;
                btnSwitchOn[i].Left = 8 + i * 32;
                btnSwitchOn[i].Top = 16;
                btnSwitchOn[i].Text = (i+1).ToString();
                btnSwitchOn[i].Tag = i + 1;
                btnSwitchOn[i].Click += new System.EventHandler(this.btnSwitchOnClick);

                btnSwitchOff[i] = new Button();
                btnSwitchOff[i].Parent = grpOnOff;
                btnSwitchOff[i].Width = 25;
                btnSwitchOff[i].Height = 25;
                btnSwitchOff[i].Left = 8 + i * 32;
                btnSwitchOff[i].Top = 40;
                btnSwitchOff[i].Text = "0";
                btnSwitchOff[i].Tag = i + 1;
                btnSwitchOff[i].Click += new System.EventHandler(this.btnSwitchOffClick);
                btnSwitchOff[i].BringToFront();

                btnToggle[i] = new Button();
                btnToggle[i].Parent = grpToggle;
                btnToggle[i].Width = 25;
                btnToggle[i].Height = 25;
                btnToggle[i].Left = 8 + i * 32;
                btnToggle[i].Top = 16;
                btnToggle[i].Text = (i + 1).ToString();
                btnToggle[i].Tag = i + 1;
                btnToggle[i].Click += new System.EventHandler(this.btnToggleClick);

                btnMomentary[i] = new Button();
                btnMomentary[i].Parent = grpMomentary;
                btnMomentary[i].Width = 25;
                btnMomentary[i].Height = 25;
                btnMomentary[i].Left = 8 + i * 32;
                btnMomentary[i].Top = 16;
                btnMomentary[i].Text = (i + 1).ToString();
                btnMomentary[i].Tag = i + 1;
                btnMomentary[i].MouseDown += new System.Windows.Forms.MouseEventHandler(this.btnMomentaryMouseDown);
                btnMomentary[i].MouseUp += new System.Windows.Forms.MouseEventHandler(this.btnMomentaryMouseUp);
            }

            updateConnectionState(false);
        }

        private void btnSetAddress_Click(object sender, EventArgs e)
        {
            if (0 != K8074Native.SetAddress(m_device, (Byte)(cbAddress.SelectedIndex+1), (Byte)(cbNewAddress.SelectedIndex+1)))
                cbAddress.SelectedIndex = cbNewAddress.SelectedIndex;

        }

        private void btnSetAll_Click(object sender, EventArgs e)
        {
            K8074Native.SendByte(m_device, (Byte)(cbAddress.SelectedIndex+1), 0xFF);
        }

        private void btnClearAll_Click(object sender, EventArgs e)
        {
            K8074Native.SendByte(m_device, (Byte)(cbAddress.SelectedIndex + 1), 0x00);
        }

        private void btnToggleClick(object sender, EventArgs e)
        {
            K8074Native.ToggleRelay(m_device, (Byte)(cbAddress.SelectedIndex + 1), (int)((Button)sender).Tag);
        }

        private void btnSwitchOnClick(object sender, EventArgs e)
        {
            K8074Native.SetRelay(m_device, (Byte)(cbAddress.SelectedIndex + 1), (int)((Button)sender).Tag);
        }

        private void btnSwitchOffClick(object sender, EventArgs e)
        {
            K8074Native.ClearRelay(m_device, (Byte)(cbAddress.SelectedIndex + 1), (int)((Button)sender).Tag);
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            K8074Native.CloseDevice(m_device);
        }

        private void btnShowAddr_Click(object sender, EventArgs e)
        {
            K8074Native.DisplayAddress(m_device);
        }

        private void btnMomentaryMouseDown(object sender, MouseEventArgs e)
        {
            m_MomIdx = (int)(((Button)sender).Tag);
            TimerMom.Enabled = true;
        }

        private void btnMomentaryMouseUp(object sender, MouseEventArgs e)
        {
            TimerMom.Enabled = false;
        }

        private void TimerMom_Tick(object sender, EventArgs e)
        {
            K8074Native.SendCommand(m_device, (Byte)(cbAddress.SelectedIndex + 1), 0x50, (Byte)(48+m_MomIdx));
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure? Note that all 8 channel relay cards under power and within range will be reset to address 1", "Reset to addr. 1", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
            {
                K8074Native.ForceAddressReset(m_device);
                cbAddress.SelectedIndex = 0;
            }
        }
    }
}