#include "K8074.h"

HANDLE WINAPI OpenDevice(LPCSTR szPort)
{
	HANDLE hDevice = NULL;
	DCB    dcb;

	hDevice = CreateFile( /* overlapped IO */
		szPort,
		GENERIC_WRITE,
		0,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL
	);

	if (INVALID_HANDLE_VALUE == hDevice)
		return FALSE;

	ZeroMemory(&dcb, sizeof(dcb));
	dcb.DCBlength = sizeof(dcb);
	if (!BuildCommDCB("2400,n,8,1", &dcb)) {
		CloseHandle(hDevice);
		return FALSE;
	}

	if (!SetCommState(hDevice, &dcb)) {
		CloseHandle(hDevice);
		return FALSE;
	}

	return hDevice;
}

BOOL WINAPI SendCommand(HANDLE hDevice, BYTE address, BYTE cmd, BYTE param)
{
	K8074_REQUEST   request;
	LPK8074_PACKET  lpPacket;
	DWORD dwWritten = 0;
	BOOL fResult;
	int i;

	lpPacket = &(request.packet[0]);

	lpPacket->bSTX      = STX;
	lpPacket->bAddress  = address;
	lpPacket->bCmd      = cmd;
	lpPacket->bParam    = param;
	lpPacket->bChecksum = - (lpPacket->bSTX + lpPacket->bAddress + lpPacket->bCmd + lpPacket->bParam);

	for(i=1; i<6; i++) {
		request.packet[i] = request.packet[0];
	}

	fResult = WriteFile(hDevice, (LPVOID)&request, sizeof(request), &dwWritten, NULL);

	return fResult;
}


BOOL WINAPI EmergencyStop(HANDLE hDevice)
{
	return SendCommand(hDevice, PARAM_EMPTY, K8074_EMERGENCY_STOP, PARAM_EMPTY);
}

BOOL WINAPI DisplayAddress(HANDLE hDevice)
{
	return SendCommand(hDevice, PARAM_EMPTY, K8074_DISPLAY_ADDRESS, PARAM_EMPTY);
}

BOOL WINAPI SendChannelCmd(HANDLE hDevice, BYTE address, BYTE command, DWORD dwRelayId)
{
	if (dwRelayId < 1 || dwRelayId > 8)
		return FALSE;

	dwRelayId += '0';

	return SendCommand(hDevice, address, command, (BYTE)(dwRelayId & 0xFF));
}

BOOL WINAPI SetRelay(HANDLE hDevice, BYTE address, DWORD dwRelayId)
{
	return SendChannelCmd(hDevice, address, K8074_SET_RELAY, dwRelayId);
}

BOOL WINAPI ClearRelay(HANDLE hDevice, BYTE address, DWORD dwRelayId)
{
	return SendChannelCmd(hDevice, address, K8074_CLEAR_RELAY, dwRelayId);
}

BOOL WINAPI ToggleRelay(HANDLE hDevice, BYTE address, DWORD dwRelayId)
{
	return SendChannelCmd(hDevice, address, K8074_TOGGLE_RELAY, dwRelayId);
}

BOOL WINAPI SetAddress(HANDLE hDevice, BYTE oldAddress, BYTE newAddress)
{
	return SendCommand(hDevice, oldAddress, K8074_SET_ADDRESS, newAddress);
}

BOOL WINAPI ForceAddressReset(HANDLE hDevice)
{
	return SendCommand(hDevice, PARAM_EMPTY, K8074_FORCE_RESET, PARAM_EMPTY);
}

BOOL WINAPI SendByte(HANDLE hDevice, BYTE address, BYTE bMask)
{
	return SendCommand(hDevice, address, K8074_SEND_BYTE, bMask);
}

VOID WINAPI CloseDevice(HANDLE hDevice)
{
	CloseHandle(hDevice);
}

DWORD WINAPI GetDllVersion(VOID)
{
	return MAKELONG(1, 0);
}